# -*- coding: utf-8 -*-

"""
Author: Zhou Zhi-Jian
Institution: Hunan University
Email: zjzhou@hnu.edu.cn
Copyright：Copyright (c) Zhou Zhi-Jian
Time: 2022/5/20 20:09

"""


plt_corlor = [
"#fdb462",
"#b3de69",
"#fccde5",
"#d9d9d9",
"#e5d8bd",
"#ccebc5",
"#694d9f",
"#80b1d3",
"#bebada",
"#8dd3c7",
"#fb8072",
"olivedrab",
"darkorange",
"darkcyan",
"darkblue",
"dodgerblue",
"lightgreen",
"orangered",
"royalblue",
"indigo",
"forestgreen",
"lightsteelblue",
"burlywood",
"antiquewhite",
"limegreen",
"darkgreen",
"darkmagenta",
"cornflowerblue",
"grey",
"tan",
"slategrey",
"ghostwhite",
"navajowhite",
"lavender",
"darkgrey",
"blanchedalmond",
"lime",
"midnightblue",
"darkgray",
"papayawhip",
"seagreen",
"navy",
"silver",
"moccasin",
"mediumseagreen",
"lightgray",
"springgreen",
"mediumblue",
"lightgrey",
"wheat",
"mintcream",
"blue",
"gainsboro",
"oldlace",
"mediumspringgreen",
"whitesmoke",
"floralwhite",
"mediumaquamarine",
"slateblue",
"darkgoldenrod",
"aquamarine",
"darkslateblue",
"goldenrod",
"turquoise",
"mediumslateblue",
"snow",
"cornsilk",
"lightseagreen",
"mediumpurple",
"lightcoral",
"rosybrown",
"gold",
"lemonchiffon",
"azure",
"mediumturquoise",
"blueviolet",
"indianred",
"khaki",
"lightcyan",
"darkorchid",
"brown",
"palegoldenrod",
"paleturquoise",
"darkviolet",
"darkkhaki",
"darkslategray",
"mediumorchid",
"ivory",
"darkslategrey",
"thistle",
"darkred",
"beige",
"teal",
"plum",
"lightyellow",
"lightgoldenrodyellow",
"purple",
"mistyrose",
"olive",
"cyan",
"salmon",
"aqua",
"m",
"tomato",
"yellow",
"darkturquoise",
"fuchsia",
"darksalmon",
"cadetblue",
"magenta",
"coral",
"yellowgreen",
"powderblue",
"orchid",
"darkolivegreen",
"lightblue",
"mediumvioletred",
"lightsalmon",
"greenyellow",
"deepskyblue",
"deeppink",
"chartreuse",
"skyblue",
"hotpink",
"seashell",
"lawngreen",
"lightskyblue",
"lavenderblush",
"sage",
"steelblue",
"palevioletred",
"saddlebrown",
"lightsage",
"aliceblue",
"crimson",
"pink",
"sandybrown",
"peachpuff",
"honeydew",
"lightslategrey",
"lightpink",
"peru",
"darkseagreen",
"lightslategray",
"linen",
"palegreen",
"slategray",
"dimgray",
"dimgrey",
"bisque",
"chocolate",
"#1F77B4",
"firebrick",
"green",
"orange",
"#9467BD",
"sienna",
"violet",
"gray",
"y",
"#17BECF",
"#8dd3c7",
]